#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	flat ivec4 BoneIDs0;
	vec4 Weights0;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture1;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct SpotLight {
	vec3 position;
	vec3 direction;
	float cutOff;
	float outerCutOff;
} spotL;

uniform vec3 shader_mode;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture1, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 1); // ambient

	
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	if (shader_mode.x != 0) {
		float theta = dot(normalize(camera_world - vert.position_world), normalize(-spotL.direction));
		float epsilon   = spotL.cutOff - spotL.outerCutOff;
		float intensity = clamp((theta - spotL.outerCutOff) / epsilon, 0.0, 1.0);  
    
		if(theta > spotL.outerCutOff) 
		{       
		  color.rgb += intensity * phong(n, camera_world - vert.position_world, v, vec3(6.0f) * texColor, materialCoefficients.y, vec3(1.0f), materialCoefficients.z, specularAlpha, true, vec3(1.0f, 0.0001f, 0.0001f));
		}
	}
	//debugSkinningColoring
	//for (int i = 0 ; i < 4 ; i++) {
    //    if (vert.BoneIDs0[i] == 2) {
    //       if (vert.Weights0[i] >= 0.7) {
    //           color = vec4(1.0, 0.0, 0.0, 0.0) * vert.Weights0[i];
    //       } else if (vert.Weights0[i] >= 0.4 && vert.Weights0[i] < 0.7) {
    //           color = vec4(0.0, 1.0, 0.0, 0.0) * vert.Weights0[i];
    //       } else if (vert.Weights0[i] >= 0.1) {
    //           color = vec4(1.0, 1.0, 0.0, 0.0) * vert.Weights0[i];
    //       }
	//
    //    }
    //}
}

