#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	mat3 TBN;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture1;
uniform sampler2D specularTexture1;
uniform sampler2D normalTexture1;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct SpotLight {
	vec3 position;
	vec3 direction;
	float cutOff;
	float outerCutOff;
} spotL;

uniform vec3 shader_mode;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	//transparent parts discard
	if (texture(diffuseTexture1, vert.uv).a < 0.6) {
		discard;
	}

	vec3 texColor = texture(diffuseTexture1, vert.uv).rgb;

	//Specular
	vec3 texSpecular = vec3(1.0f);
	if (shader_mode.y != 0) {
		
		texSpecular = texture(specularTexture1, vert.uv).rgb;
	}
	
	vec3 texNormal = texture(normalTexture1, vert.uv).rgb;
	texNormal = texNormal * 2.0 - 1.0;

	if (shader_mode.z != 0) {
		n = normalize(vert.TBN * texNormal);
	}
	

	// ambient
	color = vec4(texColor * materialCoefficients.x, 1);

	
	// add directional light contribution
	color.rgb += texSpecular * phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	if (shader_mode.x != 0) {
		float theta = dot(normalize(camera_world - vert.position_world), normalize(-spotL.direction));
		float epsilon   = spotL.cutOff - spotL.outerCutOff;
		float intensity = clamp((theta - spotL.outerCutOff) / epsilon, 0.0, 1.0);  
    
		if(theta > spotL.outerCutOff) {       
			color.rgb += texSpecular * intensity * phong(n, camera_world - vert.position_world, v, vec3(25.0f) * texColor, materialCoefficients.y, vec3(1.0f), materialCoefficients.z, specularAlpha, true, vec3(1.0f, 0.0001f, 0.0001f));
		}
	}

}

